@@ -4,7 +4,7 @@
 //!
 //! ```toml,ignore
 //! [dependencies]
-//! printpdf = "0.3.2"
+//! printpdf = "0.5.0"
 //! ```
 //!
 //! # Features
@@ -134,12 +134,12 @@
 //!     // currently, the only reliable file formats are bmp/jpeg/png
 //!     // this is an issue of the image library, not a fault of printpdf
 //!     let mut image_file = File::open("assets/img/BMP_test.bmp").unwrap();
-//!     let image = Image::try_from(image::bmp::BmpDecoder::new(&mut image_file).unwrap()).unwrap();
+//!     let image = Image::try_from(image_crate::bmp::BmpDecoder::new(&mut image_file).unwrap()).unwrap();
 //!
 //!     // translate x, translate y, rotate, scale x, scale y
 //!     // by default, an image is optimized to 300 DPI (if scale is None)
 //!     // rotations and translations are always in relation to the lower left corner
-//!     image.add_to_layer(current_layer.clone(), None, None, None, None, None, None);
+//!     image.add_to_layer(current_layer.clone(), ImageTransform::default());
 //!
 //!     // you can also construct images manually from your data:
 //!     let mut image_file_2 = ImageXObject {
@@ -331,79 +331,194 @@
 #![cfg_attr(feature="clippy", warn(wrong_pub_self_convention))]
 
 #[cfg(feature = "logging")]
-#[macro_use] pub extern crate log;
-
+#[macro_use]
+pub extern crate log;
+#[cfg(feature = "embedded_images")]
+pub extern crate image as image_crate;
 pub extern crate lopdf;
 extern crate owned_ttf_parser;
+#[cfg(feature = "svg")]
+extern crate pdf_writer;
 extern crate time;
 #[cfg(all(target_arch = "wasm32", target_os = "unknown"))]
 extern crate js_sys;
 
+pub mod color;
+pub mod ctm;
 pub mod date;
-#[cfg(feature = "embedded_images")]
-pub extern crate image;
-
-pub mod types;
-pub mod scale;
+pub mod document_info;
 pub mod errors;
-pub mod utils;
-mod glob_defines;
+pub mod extgstate;
+pub mod font;
+pub mod icc_profile;
+pub mod image;
 pub mod indices;
+pub mod line;
+pub mod ocg;
+pub mod pattern;
+pub mod pdf_conformance;
+pub mod pdf_document;
+pub mod pdf_layer;
+pub mod pdf_metadata;
+pub mod pdf_page;
+pub mod pdf_resources;
+pub mod point;
+pub mod scale;
+#[cfg(feature = "svg")]
+pub mod svg;
+pub mod utils;
+pub mod xmp_metadata;
+pub mod xobject;
 
-pub use glob_defines::*;
+pub(crate) mod glob_defines {
 
-pub use self::errors::Error;
-pub use self::errors::PdfError;
-pub use date::*;
-pub use self::errors::IndexError;
+    // Included files
+    pub const ICC_PROFILE_ECI_V2: &[u8] = include_bytes!("../assets/CoatedFOGRA39.icc");
 
-pub use self::scale::{Mm, Pt, Px};
-pub use self::types::pdf_conformance::{CustomPdfConformance, PdfConformance};
-pub use self::types::pdf_document::{PdfDocumentReference, PdfDocument};
-pub use self::types::pdf_metadata::PdfMetadata;
-pub use self::types::pdf_page::{PdfPage, PdfPageReference};
-pub use self::types::pdf_layer::{PdfLayer, PdfLayerReference};
+    /// ## General graphics state
 
-pub use self::types::plugins::xmp::xmp_metadata::XmpMetadata;
-pub use self::types::plugins::misc::document_info::DocumentInfo;
+    /// Set line width
+    pub const OP_PATH_STATE_SET_LINE_WIDTH: &str                 = "w";
+    /// Set line join
+    pub const OP_PATH_STATE_SET_LINE_JOIN: &str                  = "J";
+    /// Set line cap
+    pub const OP_PATH_STATE_SET_LINE_CAP: &str                   = "j";
+    /// Set miter limit
+    pub const OP_PATH_STATE_SET_MITER_LIMIT: &str                = "M";
+    /// Set line dash pattern
+    pub const OP_PATH_STATE_SET_LINE_DASH: &str                  = "d";
+    /// Set rendering intent
+    pub const OP_PATH_STATE_SET_RENDERING_INTENT: &str           = "ri";
+    /// Set flatness tolerance
+    pub const OP_PATH_STATE_SET_FLATNESS_TOLERANCE: &str         = "i";
+    /// (PDF 1.2) Set graphics state from parameter dictionary
+    pub const OP_PATH_STATE_SET_GS_FROM_PARAM_DICT: &str         = "gs";
 
-/// Stub module for 3D content in a PDF
-pub use self::types::plugins::graphics::three_dimensional;
-pub use self::types::plugins::graphics::two_dimensional::font::{
-    Font, FontData, BuiltinFont, ExternalFont, TextRenderingMode, IndirectFontRef, DirectFontRef, FontList
-};
-pub use self::types::plugins::graphics::two_dimensional::image::Image;
-pub use self::types::plugins::graphics::two_dimensional::line::Line;
-pub use self::types::plugins::graphics::two_dimensional::point::Point;
 
-pub use self::types::plugins::graphics::color::{
-    Color, Rgb, Cmyk, Greyscale, SpotColor, PdfColor, ColorSpace, ColorBits
-};
-pub use self::types::plugins::graphics::ctm::{CurTransMat, TextMatrix};
-pub use self::types::plugins::graphics::extgstate::{
-    ExtendedGraphicsState, ExtendedGraphicsStateList, ExtendedGraphicsStateRef, ExtendedGraphicsStateBuilder,
-    OverprintMode, BlackGenerationFunction, BlackGenerationExtraFunction, UnderColorRemovalFunction,
-    UnderColorRemovalExtraFunction, TransferFunction, TransferExtraFunction, HalftoneType,
-    SpotFunction, BlendMode, SeperableBlendMode, NonSeperableBlendMode, RenderingIntent, SoftMask,
-    SoftMaskFunction, LineJoinStyle, LineCapStyle, LineDashPattern,
-};
-pub use self::types::plugins::graphics::icc_profile::{
-    IccProfileType, IccProfile, IccProfileRef, IccProfileList
-};
-pub use self::types::plugins::graphics::ocg::{OCGList, OCGRef};
-pub use self::types::plugins::graphics::pattern::{Pattern, PatternRef, PatternList};
-pub use self::types::plugins::graphics::pdf_resources::PdfResources;
-pub use self::types::plugins::graphics::xobject::{
-    XObject, XObjectList, XObjectRef, ImageXObject, ImageXObjectRef,
-    ImageFilter, FormXObject, FormXObjectRef, FormType, SMask, GroupXObject,
-    GroupXObjectType, ReferenceXObject, OptionalContentGroup, OCGIntent, PostScriptXObject,
-};
+    /// ## Color
 
-/// Stub module for future audio embedding implementation
-pub use self::types::plugins::media::audio;
-/// Stub module for future video embedding implementation
-pub use self::types::plugins::media::video;
-/// Stub module for interactive (JavaScript) content, embedded in PDF files
-pub use self::types::plugins::interactive;
-/// Stub module for encryption (passwords). Not implemented yet.
-pub use self::types::plugins::security;
+    /// stroking color space (PDF 1.1)
+    pub const OP_COLOR_SET_STROKE_CS: &str                       = "CS";
+    /// non-stroking color space (PDF 1.1)
+    pub const OP_COLOR_SET_FILL_CS: &str                         = "cs";
+    /// set stroking color (PDF 1.1)
+    pub const OP_COLOR_SET_STROKE_COLOR: &str                    = "SC";
+    /// set stroking color (PDF 1.2) with support for ICC, etc.
+    pub const OP_COLOR_SET_STROKE_COLOR_ICC: &str                = "SCN";
+    /// set fill color (PDF 1.1)
+    pub const OP_COLOR_SET_FILL_COLOR: &str                      = "sc";
+    /// set fill color (PDF 1.2) with support for Icc, etc.
+    pub const OP_COLOR_SET_FILL_COLOR_ICC: &str                  = "scn";
+
+    /// Set the stroking color space to DeviceGray
+    pub const OP_COLOR_SET_STROKE_CS_DEVICEGRAY: &str            = "G";
+    /// Set the fill color space to DeviceGray
+    pub const OP_COLOR_SET_FILL_CS_DEVICEGRAY: &str              = "g";
+    /// Set the stroking color space to DeviceRGB
+    pub const OP_COLOR_SET_STROKE_CS_DEVICERGB: &str             = "RG";
+    /// Set the fill color space to DeviceRGB
+    pub const OP_COLOR_SET_FILL_CS_DEVICERGB: &str               = "rg";
+    /// Set the stroking color space to DeviceCMYK
+    pub const OP_COLOR_SET_STROKE_CS_DEVICECMYK: &str            = "K";
+    /// Set the fill color to DeviceCMYK
+    pub const OP_COLOR_SET_FILL_CS_DEVICECMYK: &str              = "k";
+
+
+    /// Path construction
+
+    /// Move to point
+    pub const OP_PATH_CONST_MOVE_TO: &str                        = "m";
+    /// Straight line to the two following points
+    pub const OP_PATH_CONST_LINE_TO: &str                        = "l";
+    /// Cubic bezier over four following points
+    pub const OP_PATH_CONST_4BEZIER: &str                        = "c";
+    /// Cubic bezier with two points in v1
+    pub const OP_PATH_CONST_3BEZIER_V1: &str                     = "v";
+    /// Cubic bezier with two points in v2
+    pub const OP_PATH_CONST_3BEZIER_V2: &str                     = "y";
+    /// Add rectangle to the path (width / height): x y width height re
+    pub const OP_PATH_CONST_RECT: &str                           = "re";
+    /// Close current sub-path (for appending custom patterns along line)
+    pub const OP_PATH_CONST_CLOSE_SUBPATH: &str                  = "h";
+    /// Current path is a clip path, non-zero winding order (usually in like `h W S`)
+    pub const OP_PATH_CONST_CLIP_NZ: &str = "W";
+    /// Current path is a clip path, non-zero winding order
+    pub const OP_PATH_CONST_CLIP_EO: &str = "W*";
+
+    /// Path painting
+
+    /// Stroke path
+    pub const OP_PATH_PAINT_STROKE: &str                         = "S";
+    /// Close and stroke path
+    pub const OP_PATH_PAINT_STROKE_CLOSE: &str                   = "s";
+    /// Fill path using nonzero winding number rule
+    pub const OP_PATH_PAINT_FILL_NZ: &str                        = "f";
+    /// Fill path using nonzero winding number rule (obsolete)
+    pub const OP_PATH_PAINT_FILL_NZ_OLD: &str                    = "F";
+    /// Fill path using even-odd rule
+    pub const OP_PATH_PAINT_FILL_EO: &str                        = "f*";
+    /// Fill and stroke path using nonzero winding number rule
+    pub const OP_PATH_PAINT_FILL_STROKE_NZ: &str                 = "B";
+    /// Close, fill and stroke path using nonzero winding number rule
+    pub const OP_PATH_PAINT_FILL_STROKE_CLOSE_NZ: &str           = "b";
+    /// Fill and stroke path using even-odd rule
+    pub const OP_PATH_PAINT_FILL_STROKE_EO: &str                 = "B*";
+    /// Close, fill and stroke path using even odd rule
+    pub const OP_PATH_PAINT_FILL_STROKE_CLOSE_EO: &str           = "b*";
+    /// End path without filling or stroking
+    pub const OP_PATH_PAINT_END: &str                            = "n";
+}
+
+#[doc(inline)]
+pub use crate::color::*;
+#[doc(inline)]
+pub use crate::ctm::*;
+#[doc(inline)]
+pub use crate::date::*;
+#[doc(inline)]
+pub use crate::document_info::*;
+#[doc(inline)]
+pub use crate::errors::*;
+#[doc(inline)]
+pub use crate::extgstate::*;
+#[doc(inline)]
+pub use crate::font::*;
+#[doc(inline)]
+pub use crate::glob_defines::*;
+#[doc(inline)]
+pub use crate::icc_profile::*;
+#[doc(inline)]
+pub use crate::image::*;
+#[doc(inline)]
+pub use crate::indices::*;
+#[doc(inline)]
+pub use crate::line::*;
+#[doc(inline)]
+pub use crate::ocg::*;
+#[doc(inline)]
+pub use crate::pattern::*;
+#[doc(inline)]
+pub use crate::pdf_conformance::*;
+#[doc(inline)]
+pub use crate::pdf_document::*;
+#[doc(inline)]
+pub use crate::pdf_layer::*;
+#[doc(inline)]
+pub use crate::pdf_metadata::*;
+#[doc(inline)]
+pub use crate::pdf_page::*;
+#[doc(inline)]
+pub use crate::pdf_resources::*;
+#[doc(inline)]
+pub use crate::point::*;
+#[doc(inline)]
+pub use crate::scale::*;
+#[cfg(feature = "svg")]
+#[doc(inline)]
+pub use crate::svg::*;
+#[doc(inline)]
+pub use crate::utils::*;
+#[doc(inline)]
+pub use crate::xmp_metadata::*;
+#[doc(inline)]
+pub use crate::xobject::*;
\ No newline at end of file
